@@ -4,7 +4,7 @@
  * GChemPaint library
  * molecule.cc 
  *
- * Copyright (C) 2001-2006 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2007 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -29,6 +29,7 @@
 #include "application.h"
 #include "tool.h"
 #include "stringdlg.h"
+#include <glib/gi18n-lib.h>
 #include <math.h>
 #include <unistd.h>
 #include <locale.h>
@@ -57,7 +58,12 @@ static void do_build_smiles (gcpMolecule* pMol)
 
 static void do_show_webbook (gcpMolecule* pMol)
 {
-	pMol->ShowWebbook ();
+	pMol->ShowWebBase ("http://webbook.nist.gov/cgi/cbook.cgi?Name=", "&Units=SI");
+}
+
+static void do_show_pubchem (gcpMolecule* pMol)
+{
+	pMol->ShowWebBase ("http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?CMD=search&DB=pccompound&term=\"", "\"");
 }
 
 static void do_open_in_calc (gcpMolecule* pMol)
@@ -76,6 +82,7 @@ gcpMolecule::gcpMolecule(gcpAtom* pAtom): Object(MoleculeType)
 	AddAtom(pAtom);
 	gcpChain* pChain = new gcpChain(this, pAtom); //will find the cycles
 	delete pChain;
+	m_Alignment = NULL;
 	m_Changed = true;
 }
 
@@ -183,7 +190,7 @@ bool gcpMolecule::Merge(gcpMolecule* pMolecule, bool RemoveDuplicates)
 				ma->pAtom = pAtom;
 				endj = AtomMap.end ();
 				for (j = AtomMap.begin (); j != endj; j++) {
-					if (b1 = (gcpBond*) pAtom->GetBond ((*j).second->pAtom)) {
+					if ((b1 = (gcpBond*) pAtom->GetBond ((*j).second->pAtom))) {
 						b0 = (gcpBond*) (*i)->GetBond ((*j).first);
 						if (b0) {
 							ma->sb++;
@@ -371,7 +378,6 @@ bool gcpMolecule::Load (xmlNodePtr node)
 		if (pDoc)
 			pDoc->AddFragment ((gcpFragment*) pObject);
 		AddFragment ((gcpFragment*) pObject);
-		((gcpFragment*) pObject)->AnalContent ();
 		child = GetNextNodeByName (child->next, "fragment");
 	}
 
@@ -463,7 +469,7 @@ Object* gcpMolecule::GetAtomAt(double x, double y, double z)
 	std::list<gcpBond*>::iterator n, end = m_Bonds.end();
 	Object* pObj = NULL;
 	for (n = m_Bonds.begin(); n != end; n++)
-		if (pObj = (*n)->GetAtomAt (x, y))
+		if ((pObj = (*n)->GetAtomAt (x, y)))
 			break;
 	return pObj;
 }
@@ -515,6 +521,10 @@ bool gcpMolecule::BuildContextualMenu (GtkUIManager *UIManager, Object *object)
 			g_signal_connect_swapped (action, "activate", G_CALLBACK (do_show_webbook), this);
 			gtk_action_group_add_action (group, action);
 			gtk_ui_manager_add_ui_from_string (UIManager, "<ui><popup><menu action='Molecule'><menuitem action='webbook'/></menu></popup></ui>", -1, NULL);
+			action = gtk_action_new ("pubchem", _("PubChem page for this molecule"), NULL, NULL);
+			g_signal_connect_swapped (action, "activate", G_CALLBACK (do_show_pubchem), this);
+			gtk_action_group_add_action (group, action);
+			gtk_ui_manager_add_ui_from_string (UIManager, "<ui><popup><menu action='Molecule'><menuitem action='pubchem'/></menu></popup></ui>", -1, NULL);
 		}
 		action = gtk_action_new ("smiles", _("Generate Smiles"), NULL, NULL);
 		g_signal_connect_swapped (action, "activate", G_CALLBACK (do_build_smiles), this);
@@ -715,7 +725,7 @@ void gcpMolecule::BuildInChI ()
 		setlocale (LC_NUMERIC, old_num_locale);
 		g_free (old_num_locale);
 		// remove "INCHI=" and the new line char at the end
-		m_InChI = ofs.str ().substr (6, ofs.str ().length () - 8);
+		m_InChI = ofs.str ().substr (0, ofs.str ().length () - 2);
 	} else {
 		Conv.SetInAndOutFormats (pMolFormat, pMolFormat);
 		char *tmpname = g_strdup ("/tmp/inchiXXXXXX");
@@ -773,7 +783,7 @@ void gcpMolecule::ShowInChI ()
 	new gcpStringDlg (reinterpret_cast<gcpDocument *>(GetDocument ()), m_InChI, gcpStringDlg::INCHI);
 }
 
-void gcpMolecule::ShowWebbook ()
+void gcpMolecule::ShowWebBase (char const* uri_start, char const *uri_end)
 {
 	if (m_Changed)
 		BuildInChI ();
@@ -782,7 +792,7 @@ void gcpMolecule::ShowWebbook ()
 	string::size_type t;
 	while ((t = m_InChI.find ('+')) != string::npos)
 		m_InChI.replace (t, 1, "%2b");
-	string uri = string ("http://webbook.nist.gov/cgi/cbook.cgi?Name=") + m_InChI.c_str () + string ("&Units=SI");
+	string uri = string (uri_start) + m_InChI + uri_end;
 	((gcpDocument*) GetDocument ())->GetApplication ()->ShowURI (uri);
 }
 
@@ -819,3 +829,11 @@ void gcpMolecule::CheckCrossings (gcpBond *pBond)
 			pView->Update (*i);
 		}
 }
+
+void gcpMolecule::Add (GtkWidget* w)
+{
+	/* Now that Object::Add adds children, we need to override this method
+	because a molecule children (atoms, bonds, and fragments) are added to
+	the widget by the document. FIXME! FIXME! FIXME! CHANGE THIS OLD WEIRD CODE!
+	*/
+}
