@@ -4,9 +4,7 @@
  * GChemPaint library
  * reaction.h 
  *
- * Copyright (C) 2002-2004
- *
- * Developed by Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2002-2007 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -38,7 +36,6 @@ public:
 	gcpReaction ();
 	virtual ~gcpReaction ();
 	
-	virtual void Add ( GtkWidget* w);
 	virtual bool Build (list<Object*>& Children) throw (invalid_argument);
 	virtual void Transform2D (Matrix2D& m, double x, double y);
 	virtual bool BuildContextualMenu (GtkUIManager *UIManager, Object *object);
