@@ -4,7 +4,7 @@
  * GChemPaint arrows plugin
  * arrowtool.cc 
  *
- * Copyright (C) 2001-2006 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2007 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -35,8 +35,9 @@
 #include "lib/reaction-arrow.h"
 #include "lib/settings.h"
 #include "lib/theme.h"
-#include <math.h>
+#include <glib/gi18n-lib.h>
 #include <gconf/gconf-client.h>
+#include <math.h>
 
 static char* ToolNames[] = {
 	"SimpleArrow",
@@ -46,19 +47,6 @@ static char* ToolNames[] = {
 	"RetrosynthesisArrow"
 };
 
-
-class gcpArrowToolDlg: public Dialog
-{
-public:
-	gcpArrowToolDlg (gcpApplication* App, bool FullHeads);
-	virtual ~gcpArrowToolDlg ();
-
-	virtual bool Apply ();
-	
-private:
-	GtkToggleButton *full_btn;
-};
-
 static void on_default (GtkToggleButton *button)
 {
 	GConfClient *conf_client = gconf_client_get_default ();
@@ -66,106 +54,6 @@ static void on_default (GtkToggleButton *button)
 	g_object_unref (conf_client);
 }
 
-gcpArrowToolDlg::gcpArrowToolDlg (gcpApplication* App, bool FullHeads): Dialog(App, DATADIR"/gchempaint-unstable/ui/arrowtool.glade", "arrowtool")
-{
-	GtkTable *table = GTK_TABLE (glade_xml_get_widget (xml, "heads-table"));
-	GnomeCanvas *canvas = GNOME_CANVAS (gnome_canvas_new_aa ());
-	gcpTheme *Theme = ThemeManager.GetTheme ("Default");
-	double width = (Theme->GetArrowLength () + 2 * Theme->GetArrowPadding ()) * Theme->GetZoomFactor (),
-		height = (Theme->GetArrowDist () + Theme->GetArrowWidth () + 2 * (Theme->GetArrowHeadB () + Theme->GetPadding ())) * Theme->GetZoomFactor ();
-	gtk_widget_set_size_request (GTK_WIDGET (canvas), (int) width, (int) height);
-	GnomeCanvasGroup *group = gnome_canvas_root (canvas);
-	GnomeCanvasPoints *points = gnome_canvas_points_new (2);
-	gnome_canvas_set_scroll_region(canvas, 0, 0, Theme ->GetArrowLength (), Theme->GetArrowDist () + Theme->GetArrowWidth () + 2 * Theme->GetArrowHeadB ());
-	points->coords[0] = (width - Theme->GetArrowLength () * Theme->GetZoomFactor ()) / 2.;
-	points->coords[1] = points->coords[3] = (height - Theme->GetArrowDist () * Theme->GetZoomFactor ()) / 2.;
-	points->coords[2] = (width + Theme ->GetArrowLength () * Theme->GetZoomFactor ()) / 2.;
-	gnome_canvas_item_new (
-						group,
-						gnome_canvas_line_ext_get_type (),
-						"points", points,
-						"fill_color", "black",
-						"width_units", Theme->GetArrowWidth () * Theme->GetZoomFactor (),
-						"last_arrowhead", true,
-						"arrow_shape_a",Theme->GetArrowHeadA () * Theme->GetZoomFactor (),
-						"arrow_shape_b",Theme->GetArrowHeadB () * Theme->GetZoomFactor (),
-						"arrow_shape_c",Theme->GetArrowHeadC () * Theme->GetZoomFactor (),
-						"last_arrowhead_style", (unsigned char)ARROW_HEAD_LEFT,
-						NULL);
-	points->coords[0] = (width + Theme->GetArrowLength () * Theme->GetZoomFactor ()) / 2.;
-	points->coords[1] = points->coords[3] = (height + Theme->GetArrowDist () * Theme->GetZoomFactor ()) / 2.;
-	points->coords[2] = (width - Theme->GetArrowLength () * Theme->GetZoomFactor ()) / 2.;
-	gnome_canvas_item_new (
-						group,
-						gnome_canvas_line_ext_get_type (),
-						"points", points,
-						"fill_color", "black",
-						"width_units",Theme->GetArrowWidth () * Theme->GetZoomFactor (),
-						"last_arrowhead", true,
-						"arrow_shape_a",Theme->GetArrowHeadA () * Theme->GetZoomFactor (),
-						"arrow_shape_b",Theme->GetArrowHeadB () * Theme->GetZoomFactor (),
-						"arrow_shape_c",Theme->GetArrowHeadC  ()* Theme->GetZoomFactor (),
-						"last_arrowhead_style", (unsigned char)ARROW_HEAD_LEFT,
-						NULL);
-	gtk_table_attach (table, GTK_WIDGET (canvas), 1, 2, 0, 1, GTK_FILL, GTK_FILL, 10, 0);
-	canvas = GNOME_CANVAS (gnome_canvas_new_aa ());
-	gtk_widget_set_size_request (GTK_WIDGET (canvas), (int) width, (int) height);
-	group = gnome_canvas_root (canvas);
-	gnome_canvas_set_scroll_region(canvas, 0, 0, Theme->GetArrowLength (), Theme->GetArrowDist () + Theme->GetArrowWidth () + 2 * Theme->GetArrowHeadB ());
-	points->coords[0] = (width - Theme->GetArrowLength () * Theme->GetZoomFactor ()) / 2.;
-	points->coords[1] = points->coords[3] = (height - Theme->GetArrowDist () * Theme->GetZoomFactor ()) / 2.;
-	points->coords[2] = (width + Theme->GetArrowLength () * Theme->GetZoomFactor ()) / 2.;
-	gnome_canvas_item_new (
-						group,
-						gnome_canvas_line_ext_get_type (),
-						"points", points,
-						"fill_color", "black",
-						"width_units", Theme->GetArrowWidth () * Theme->GetZoomFactor (),
-						"last_arrowhead", true,
-						"arrow_shape_a",Theme->GetArrowHeadA () * Theme->GetZoomFactor (),
-						"arrow_shape_b",Theme->GetArrowHeadB () * Theme->GetZoomFactor (),
-						"arrow_shape_c",Theme->GetArrowHeadC () * Theme->GetZoomFactor (),
-						"last_arrowhead_style", (unsigned char)ARROW_HEAD_BOTH,
-						NULL);
-	points->coords[0] = (width + Theme->GetArrowLength () * Theme->GetZoomFactor ()) / 2.;
-	points->coords[1] = points->coords[3] = (height + Theme->GetArrowDist () * Theme->GetZoomFactor ()) / 2.;
-	points->coords[2] = (width - Theme->GetArrowLength () * Theme->GetZoomFactor ()) / 2.;
-	gnome_canvas_item_new (
-						group,
-						gnome_canvas_line_ext_get_type (),
-						"points", points,
-						"fill_color", "black",
-						"width_units",Theme->GetArrowWidth () * Theme->GetZoomFactor (),
-						"last_arrowhead", true,
-						"arrow_shape_a",Theme->GetArrowHeadA () * Theme->GetZoomFactor (),
-						"arrow_shape_b",Theme->GetArrowHeadB () * Theme->GetZoomFactor (),
-						"arrow_shape_c",Theme->GetArrowHeadC () * Theme->GetZoomFactor (),
-						"last_arrowhead_style", (unsigned char)ARROW_HEAD_BOTH,
-						NULL);
-	gtk_table_attach (table, GTK_WIDGET (canvas), 1, 2, 1, 2, GTK_FILL, GTK_FILL, 10, 0);
-	gnome_canvas_points_free (points);
-	full_btn = GTK_TOGGLE_BUTTON (glade_xml_get_widget (xml, "full"));
-	if (FullHeads) gtk_toggle_button_set_active (full_btn, true);
-	GtkWidget* w = glade_xml_get_widget (xml, "default");
-	g_signal_connect_swapped (w, "clicked", G_CALLBACK (on_default), full_btn);
-	reinterpret_cast<gcpApplication*> (App)->SetDialog ("ArrowHeads", this);
-	gtk_widget_show_all (GTK_WIDGET (dialog));
-}
-
-gcpArrowToolDlg::~gcpArrowToolDlg ()
-{
-	reinterpret_cast<gcpApplication*> (m_App)->SetDialog ("ArrowHeads", NULL);
-}
-
-bool gcpArrowToolDlg::Apply ()
-{
-	gcpArrowTool *tool = (gcpArrowTool*)
-			reinterpret_cast<gcpApplication*> (m_App)->GetTool ("ReversibleArrow");
-	if (tool)
-		tool->SetArrowType (gtk_toggle_button_get_active (full_btn)? gcpFullReversibleArrow: gcpReversibleArrow);
-	return true;
-}
-
 gcpArrowTool::gcpArrowTool(gcpApplication* App, unsigned ArrowType): gcpTool(App, ToolNames[ArrowType])
 {
 	points = gnome_canvas_points_new(2);
@@ -183,7 +71,7 @@ bool gcpArrowTool::OnClicked()
 		gcpDocument *pDoc = m_pView->GetDoc ();
 	gcpTheme *pTheme = pDoc->GetTheme ();
 	m_y1 = m_y0;
-	m_x1 = m_x0 + pTheme->GetArrowLength () * pTheme->GetZoomFactor ();
+	m_x1 = m_x0 + pDoc->GetArrowLength () * pTheme->GetZoomFactor ();
 	switch(m_ArrowType)
 	{
 		case gcpSimpleArrow:
@@ -521,110 +409,116 @@ void gcpArrowTool::OnRelease()
 	pDoc->FinishOperation();
 }
 
-void gcpArrowTool::SetOptions ()
-{
-	if (m_ArrowType != gcpReversibleArrow && m_ArrowType != gcpFullReversibleArrow)
-		return;
-	gcpArrowToolDlg* dlg = (gcpArrowToolDlg*) m_pApp->GetDialog("ArrowHeads");
-	if (!dlg)
-	{
-		dlg = new gcpArrowToolDlg (m_pApp, m_ArrowType == gcpFullReversibleArrow);
-	}
-	else gdk_window_raise (GTK_WIDGET(dlg->GetWindow())->window); 
-}
-
 static void on_full_toggled (GtkToggleButton *button, gcpArrowTool *tool)
 {
 	tool->SetArrowType (gtk_toggle_button_get_active (button)? gcpFullReversibleArrow: gcpReversibleArrow);
 }
 
+static void on_length_changed (GtkSpinButton *btn, gcpArrowTool *tool)
+{
+	tool->SetLength (gtk_spin_button_get_value (btn));
+}
+
 GtkWidget *gcpArrowTool::GetPropertyPage ()
 {
-	if (m_ArrowType != gcpReversibleArrow && m_ArrowType != gcpFullReversibleArrow)
-		return NULL;
-	GladeXML *xml = glade_xml_new (DATADIR"/gchempaint-unstable/ui/arrowtool.glade", "heads-box", GETTEXT_PACKAGE);
-	GtkTable *table = GTK_TABLE (glade_xml_get_widget (xml, "heads-table"));
-	GnomeCanvas *canvas = GNOME_CANVAS (gnome_canvas_new_aa ());
-	gcpTheme *Theme = ThemeManager.GetTheme ("Default");
-	double width = (Theme->GetArrowLength () + 2 * Theme->GetArrowPadding ()) * Theme->GetZoomFactor (),
-		height = (Theme->GetArrowDist () + Theme->GetArrowWidth () + 2 * (Theme->GetArrowHeadB () + Theme->GetPadding ())) * Theme->GetZoomFactor ();
-	gtk_widget_set_size_request (GTK_WIDGET (canvas), (int) width, (int) height);
-	GnomeCanvasGroup *group = gnome_canvas_root (canvas);
-	GnomeCanvasPoints *points = gnome_canvas_points_new (2);
-	gnome_canvas_set_scroll_region (canvas, 0, 0, Theme->GetArrowLength (), Theme->GetArrowDist () + Theme->GetArrowWidth () + 2 * Theme->GetArrowHeadB ());
-	points->coords[0] = (width - Theme->GetArrowLength () * Theme->GetZoomFactor ()) / 2.;
-	points->coords[1] = points->coords[3] = (height - Theme->GetArrowDist  ()* Theme->GetZoomFactor ()) / 2.;
-	points->coords[2] = (width + Theme->GetArrowLength () * Theme->GetZoomFactor ()) / 2.;
-	gnome_canvas_item_new (
-						group,
-						gnome_canvas_line_ext_get_type (),
-						"points", points,
-						"fill_color", "black",
-						"width_units", Theme->GetArrowWidth () * Theme->GetZoomFactor (),
-						"last_arrowhead", true,
-						"arrow_shape_a",Theme->GetArrowHeadA () * Theme->GetZoomFactor (),
-						"arrow_shape_b",Theme->GetArrowHeadB () * Theme->GetZoomFactor (),
-						"arrow_shape_c",Theme->GetArrowHeadC () * Theme->GetZoomFactor (),
-						"last_arrowhead_style", (unsigned char)ARROW_HEAD_LEFT,
-						NULL);
-	points->coords[0] = (width + Theme->GetArrowLength () * Theme->GetZoomFactor ()) / 2.;
-	points->coords[1] = points->coords[3] = (height + Theme->GetArrowDist () * Theme->GetZoomFactor ()) / 2.;
-	points->coords[2] = (width - Theme->GetArrowLength () * Theme->GetZoomFactor ()) / 2.;
-	gnome_canvas_item_new (
-						group,
-						gnome_canvas_line_ext_get_type (),
-						"points", points,
-						"fill_color", "black",
-						"width_units",Theme->GetArrowWidth () * Theme->GetZoomFactor (),
-						"last_arrowhead", true,
-						"arrow_shape_a",Theme->GetArrowHeadA () * Theme->GetZoomFactor (),
-						"arrow_shape_b",Theme->GetArrowHeadB () * Theme->GetZoomFactor (),
-						"arrow_shape_c",Theme->GetArrowHeadC () * Theme->GetZoomFactor (),
-						"last_arrowhead_style", (unsigned char)ARROW_HEAD_LEFT,
-						NULL);
-	gtk_widget_show (GTK_WIDGET (canvas));
-	gtk_table_attach (table, GTK_WIDGET (canvas), 1, 2, 0, 1, GTK_FILL, GTK_FILL, 10, 0);
-	canvas = GNOME_CANVAS (gnome_canvas_new_aa ());
-	gtk_widget_set_size_request (GTK_WIDGET (canvas), (int) width, (int) height);
-	group = gnome_canvas_root (canvas);
-	gnome_canvas_set_scroll_region (canvas, 0, 0, Theme->GetArrowLength (), Theme->GetArrowDist () + Theme->GetArrowWidth () + 2 * Theme->GetArrowHeadB ());
-	points->coords[0] = (width - Theme->GetArrowLength () * Theme->GetZoomFactor ()) / 2.;
-	points->coords[1] = points->coords[3] = (height - Theme->GetArrowDist () * Theme->GetZoomFactor ()) / 2.;
-	points->coords[2] = (width + Theme->GetArrowLength () * Theme->GetZoomFactor ()) / 2.;
-	gnome_canvas_item_new (
-						group,
-						gnome_canvas_line_ext_get_type (),
-						"points", points,
-						"fill_color", "black",
-						"width_units", Theme->GetArrowWidth () * Theme->GetZoomFactor (),
-						"last_arrowhead", true,
-						"arrow_shape_a",Theme->GetArrowHeadA () * Theme->GetZoomFactor (),
-						"arrow_shape_b",Theme->GetArrowHeadB () * Theme->GetZoomFactor (),
-						"arrow_shape_c",Theme->GetArrowHeadC () * Theme->GetZoomFactor (),
-						"last_arrowhead_style", (unsigned char)ARROW_HEAD_BOTH,
-						NULL);
-	points->coords[0] = (width + Theme->GetArrowLength () * Theme->GetZoomFactor ()) / 2.;
-	points->coords[1] = points->coords[3] = (height + Theme->GetArrowDist () * Theme->GetZoomFactor ()) / 2.;
-	points->coords[2] = (width - Theme->GetArrowLength () * Theme->GetZoomFactor ()) / 2.;
-	gnome_canvas_item_new (
-						group,
-						gnome_canvas_line_ext_get_type (),
-						"points", points,
-						"fill_color", "black",
-						"width_units",Theme->GetArrowWidth () * Theme->GetZoomFactor (),
-						"last_arrowhead", true,
-						"arrow_shape_a",Theme->GetArrowHeadA () * Theme->GetZoomFactor (),
-						"arrow_shape_b",Theme->GetArrowHeadB () * Theme->GetZoomFactor (),
-						"arrow_shape_c",Theme->GetArrowHeadC () * Theme->GetZoomFactor (),
-						"last_arrowhead_style", (unsigned char)ARROW_HEAD_BOTH,
-						NULL);
-	gtk_widget_show (GTK_WIDGET (canvas));
-	gtk_table_attach (table, GTK_WIDGET (canvas), 1, 2, 1, 2, GTK_FILL, GTK_FILL, 10, 0);
-	gnome_canvas_points_free (points);
-	GtkWidget *b = glade_xml_get_widget (xml, "full");
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (b), m_ArrowType == gcpFullReversibleArrow);
-	g_signal_connect (G_OBJECT (b), "toggled", G_CALLBACK (on_full_toggled), this);
-	GtkWidget *w = glade_xml_get_widget (xml, "default");
-	g_signal_connect_swapped (w, "clicked", G_CALLBACK (on_default), b);
-	return glade_xml_get_widget (xml, "heads-box");
+	bool show_all = m_ArrowType == gcpReversibleArrow || m_ArrowType == gcpFullReversibleArrow;
+	GladeXML *xml = glade_xml_new (DATADIR"/gchempaint-unstable/ui/arrowtool.glade", (show_all? "arrow-box": "length-box"), GETTEXT_PACKAGE);
+	if (show_all) {
+		GtkTable *table = GTK_TABLE (glade_xml_get_widget (xml, "heads-table"));
+		GnomeCanvas *canvas = GNOME_CANVAS (gnome_canvas_new_aa ());
+		gcpTheme *Theme = ThemeManager.GetTheme ("Default");
+		double width = (Theme->GetArrowLength () * Theme->GetZoomFactor () + 2 * Theme->GetArrowPadding ()),
+			height = Theme->GetArrowDist () + Theme->GetArrowWidth () + 2 * (Theme->GetArrowHeadB () + Theme->GetPadding ());
+		gtk_widget_set_size_request (GTK_WIDGET (canvas), (int) width, (int) height);
+		GnomeCanvasGroup *group = gnome_canvas_root (canvas);
+		GnomeCanvasPoints *points = gnome_canvas_points_new (2);
+		gnome_canvas_set_scroll_region (canvas, 0, 0, Theme->GetArrowLength (), Theme->GetArrowDist () + Theme->GetArrowWidth () + 2 * Theme->GetArrowHeadB ());
+		points->coords[0] = (width - Theme->GetArrowLength () * Theme->GetZoomFactor ()) / 2.;
+		points->coords[1] = points->coords[3] = (height - Theme->GetArrowDist  ()) / 2.;
+		points->coords[2] = (width + Theme->GetArrowLength () * Theme->GetZoomFactor ()) / 2.;
+		gnome_canvas_item_new (
+							group,
+							gnome_canvas_line_ext_get_type (),
+							"points", points,
+							"fill_color", "black",
+							"width_units", Theme->GetArrowWidth (),
+							"last_arrowhead", true,
+							"arrow_shape_a",Theme->GetArrowHeadA (),
+							"arrow_shape_b",Theme->GetArrowHeadB (),
+							"arrow_shape_c",Theme->GetArrowHeadC (),
+							"last_arrowhead_style", (unsigned char)ARROW_HEAD_LEFT,
+							NULL);
+		points->coords[0] = (width + Theme->GetArrowLength () * Theme->GetZoomFactor ()) / 2.;
+		points->coords[1] = points->coords[3] = (height + Theme->GetArrowDist ()) / 2.;
+		points->coords[2] = (width - Theme->GetArrowLength () * Theme->GetZoomFactor ()) / 2.;
+		gnome_canvas_item_new (
+							group,
+							gnome_canvas_line_ext_get_type (),
+							"points", points,
+							"fill_color", "black",
+							"width_units",Theme->GetArrowWidth (),
+							"last_arrowhead", true,
+							"arrow_shape_a",Theme->GetArrowHeadA (),
+							"arrow_shape_b",Theme->GetArrowHeadB (),
+							"arrow_shape_c",Theme->GetArrowHeadC (),
+							"last_arrowhead_style", (unsigned char)ARROW_HEAD_LEFT,
+							NULL);
+		gtk_widget_show (GTK_WIDGET (canvas));
+		gtk_table_attach (table, GTK_WIDGET (canvas), 1, 2, 0, 1, GTK_FILL, GTK_FILL, 10, 0);
+		canvas = GNOME_CANVAS (gnome_canvas_new_aa ());
+		gtk_widget_set_size_request (GTK_WIDGET (canvas), (int) width, (int) height);
+		group = gnome_canvas_root (canvas);
+		gnome_canvas_set_scroll_region (canvas, 0, 0, Theme->GetArrowLength (), Theme->GetArrowDist () + Theme->GetArrowWidth () + 2 * Theme->GetArrowHeadB ());
+		points->coords[0] = (width - Theme->GetArrowLength () * Theme->GetZoomFactor ()) / 2.;
+		points->coords[1] = points->coords[3] = (height - Theme->GetArrowDist ()) / 2.;
+		points->coords[2] = (width + Theme->GetArrowLength () * Theme->GetZoomFactor ()) / 2.;
+		gnome_canvas_item_new (
+							group,
+							gnome_canvas_line_ext_get_type (),
+							"points", points,
+							"fill_color", "black",
+							"width_units", Theme->GetArrowWidth (),
+							"last_arrowhead", true,
+							"arrow_shape_a",Theme->GetArrowHeadA (),
+							"arrow_shape_b",Theme->GetArrowHeadB (),
+							"arrow_shape_c",Theme->GetArrowHeadC (),
+							"last_arrowhead_style", (unsigned char)ARROW_HEAD_BOTH,
+							NULL);
+		points->coords[0] = (width + Theme->GetArrowLength () * Theme->GetZoomFactor ()) / 2.;
+		points->coords[1] = points->coords[3] = (height + Theme->GetArrowDist ()) / 2.;
+		points->coords[2] = (width - Theme->GetArrowLength () * Theme->GetZoomFactor ()) / 2.;
+		gnome_canvas_item_new (
+							group,
+							gnome_canvas_line_ext_get_type (),
+							"points", points,
+							"fill_color", "black",
+							"width_units",Theme->GetArrowWidth (),
+							"last_arrowhead", true,
+							"arrow_shape_a",Theme->GetArrowHeadA (),
+							"arrow_shape_b",Theme->GetArrowHeadB (),
+							"arrow_shape_c",Theme->GetArrowHeadC (),
+							"last_arrowhead_style", (unsigned char)ARROW_HEAD_BOTH,
+							NULL);
+		gtk_widget_show (GTK_WIDGET (canvas));
+		gtk_table_attach (table, GTK_WIDGET (canvas), 1, 2, 1, 2, GTK_FILL, GTK_FILL, 10, 0);
+		gnome_canvas_points_free (points);
+		GtkWidget *b = glade_xml_get_widget (xml, "full");
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (b), m_ArrowType == gcpFullReversibleArrow);
+		g_signal_connect (G_OBJECT (b), "toggled", G_CALLBACK (on_full_toggled), this);
+		GtkWidget *w = glade_xml_get_widget (xml, "default");
+		g_signal_connect_swapped (w, "clicked", G_CALLBACK (on_default), b);
+	}
+	m_LengthBtn = GTK_SPIN_BUTTON (glade_xml_get_widget (xml, "arrow-length"));
+	g_signal_connect (m_LengthBtn, "value-changed", G_CALLBACK (on_length_changed), this);
+	return glade_xml_get_widget (xml, (show_all? "arrow-box": "length-box"));
+}
+
+void gcpArrowTool::SetLength (double length)
+{
+	m_pApp->GetActiveDocument ()->SetArrowLength (length);
+}
+
+void gcpArrowTool::Activate ()
+{
+	gtk_spin_button_set_value (m_LengthBtn, m_pApp->GetActiveDocument ()->GetArrowLength ());
 }
