@@ -4,9 +4,7 @@
  * GChemPaint bonds plugin
  * delocalizedtool.cc 
  *
- * Copyright (C) 2004
- *
- * Developed by Jean Bréfort <jean.brefort@ac-dijon.fr>
+ * Copyright (C) 2004-2005 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -20,10 +18,11 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
+#include "gchempaint-config.h"
 #include "delocalizedtool.h"
 
 gcpDelocalizedTool::gcpDelocalizedTool (gcpApplication *App): gcpTool (App, "DelocalizedBond")
@@ -36,6 +35,7 @@ gcpDelocalizedTool::~gcpDelocalizedTool ()
 
 bool gcpDelocalizedTool::OnClicked ()
 {
+	return false;
 }
 
 void gcpDelocalizedTool::OnDrag ()
