@@ -4,7 +4,7 @@
  * GChemPaint cycles plugin
  * cycletool.cc 
  *
- * Copyright (C) 2001-2006 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2007 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -25,10 +25,10 @@
 #include "gchempaint-config.h"
 #include "cycletool.h"
 #include "lib/settings.h"
-#include "lib/mendeleiev.h"
 #include "lib/document.h"
 #include "lib/application.h"
 #include "lib/theme.h"
+#include <glib/gi18n-lib.h>
 #include <math.h>
 #include <list>
 #include <vector>
@@ -553,6 +553,7 @@ void gcpCycleTool::OnRelease()
 	gcpDocument *pDoc = m_pView->GetDoc ();
 	gcpOperation *pOp = NULL;
 	gcpMolecule *pMol = NULL;
+	char const *MolId = NULL;
 	int i;
 	for (i = 0; i < m_size; i++)
 	{
@@ -576,6 +577,9 @@ void gcpCycleTool::OnRelease()
 				if (pMol == NULL) {
 					pMol = reinterpret_cast<gcpMolecule *> (pAtom[i]->GetMolecule ());
 					pMol->Lock (true);
+					// we must store the id, the molecule might be destroyed
+					// FIXME! what happens to the parent in such a case?
+					MolId = pMol->GetId ();
 				}
 				// if group not in ModifiedObjects, add it and save it
 				pObject = pAtom[i]->GetGroup ();
@@ -623,6 +627,8 @@ void gcpCycleTool::OnRelease()
 		pOp->AddObject (pObject);
 	}
 	pDoc->FinishOperation ();
+	if (pMol)
+		pMol = static_cast <gcpMolecule*> (pDoc->GetDescendant (MolId));
 	if (pMol) {
 		pMol->Lock (false);
 		pMol->EmitSignal (OnChangedSignal);
@@ -769,28 +775,13 @@ GtkWidget *gcpCycleTool::GetPropertyPage ()
 	return glade_xml_get_widget (xml, "cycle");
 }
 
-extern bool new_ui;
 void gcpCycleTool::Activate ()
 {
-	if (new_ui) {
-		gcpDocument *pDoc = m_pApp->GetActiveDocument ();
-		gtk_spin_button_set_value (m_LengthBtn, pDoc->GetBondLength ());
-		gtk_toggle_button_set_active (m_MergeBtn, MergeAtoms);
-	}
+	gcpDocument *pDoc = m_pApp->GetActiveDocument ();
+	gtk_spin_button_set_value (m_LengthBtn, pDoc->GetBondLength ());
+	gtk_toggle_button_set_active (m_MergeBtn, MergeAtoms);
 }
 
-class gcpCNToolDlg: public Dialog
-{
-public:
-	gcpCNToolDlg(gcpApplication* App, unsigned char size = 9);
-	virtual ~gcpCNToolDlg();
-
-	virtual bool Apply();
-	
-private:
-	GtkSpinButton* m_button;
-};
-
 gcpNCycleTool::gcpNCycleTool(gcpApplication* App, unsigned char size): gcpCycleTool(App, 9)
 {
 	SetSize(size);
@@ -826,35 +817,3 @@ GtkWidget *gcpNCycleTool::GetPropertyPage ()
 	g_signal_connect (m_SizeBtn, "value-changed", G_CALLBACK (on_size_changed), this);
 	return glade_xml_get_widget (xml, "cycle");
 }
-
-void gcpNCycleTool::SetOptions ()
-{
-	gcpCNToolDlg* dlg = (gcpCNToolDlg*) m_pApp->GetDialog ("CycleN");
-	if (!dlg)
-	{
-		dlg = new gcpCNToolDlg (m_pApp, GetSize ());
-	}
-	else gdk_window_raise (GTK_WIDGET (dlg->GetWindow ())->window); 
-}
-
-gcpCNToolDlg::gcpCNToolDlg(gcpApplication* App, unsigned char size): Dialog(App, DATADIR"/gchempaint-unstable/ui/cyclentool.glade", "cyclentool")
-{
-	m_button = GTK_SPIN_BUTTON(glade_xml_get_widget(xml, "sizebtn"));
-	gtk_spin_button_set_value(m_button, size);
-	App->SetDialog("CycleN", this);
-	gtk_widget_show(GTK_WIDGET(dialog));
-}
-
-gcpCNToolDlg::~gcpCNToolDlg ()
-{
-	reinterpret_cast<gcpApplication*> (m_App)->SetDialog ("CycleN", NULL);
-}
-
-bool gcpCNToolDlg::Apply ()
-{
-	gcpNCycleTool* tool = (gcpNCycleTool*)
-			reinterpret_cast<gcpApplication*> (m_App)->GetTool ("CycleN");
-	if (tool)
-		tool->SetSize ((unsigned char) gtk_spin_button_get_value_as_int (m_button));
-	return true;
-}
