@@ -66,6 +66,9 @@ bool gcpFragmentTool::OnClicked ()
 		pDoc->EmptyTranslationTable ();
 		m_pObject = fragment;
 	}
+	struct GnomeCanvasPangoSelBounds bounds;
+	bool need_update = false;
+	gcpFragment *pFragment = NULL;
 	if (m_pObject) {
 		switch(m_pObject->GetType ()) {
 			case AtomType: {
@@ -77,7 +80,7 @@ bool gcpFragmentTool::OnClicked ()
 				gcpMolecule *pMol = (gcpMolecule*) pAtom->GetMolecule ();
 				map<Atom*, Bond*>::iterator i;
 				gcpBond *pBond = (gcpBond*) pAtom->GetFirstBond (i);
-				gcpFragment *pFragment = new gcpFragment (x, y);
+				pFragment = new gcpFragment (x, y);
 				gcpAtom* pFragAtom = (gcpAtom*) pFragment->m_Atom;
 				map<string, Object*>::iterator ie;
 				Object* electron = pAtom->GetFirstChild (ie);
@@ -105,11 +108,13 @@ bool gcpFragmentTool::OnClicked ()
 						buf = g_strdup_printf ("H%d", n);
 					else
 						buf = g_strdup ("H");
+					bounds.start = bounds.cur = ((pAtom->GetBestSide ())? strlen (pAtom->GetSymbol ()): 0);
+					pFragment->OnSelChanged (&bounds);
 					gcp_pango_layout_replace_text (pFragment->GetLayout (),
-						((pAtom->GetBestSide ())? strlen (pAtom->GetSymbol ()): 0),
+						bounds.cur,
 						0, buf, pDoc->GetPangoAttrList ());
-					pFragment->AnalContent ();
-					pFragment->OnChanged (false);
+					bounds.cur +=  strlen (buf);
+					need_update = true;
 					g_free (buf);
 				}
 				delete pAtom;
@@ -133,6 +138,11 @@ bool gcpFragmentTool::OnClicked ()
 		}
 		m_pObject->SetSelected (m_pWidget, SelStateUpdating);
 		m_Active = GNOME_CANVAS_PANGO (g_object_get_data (G_OBJECT (m_pData->Items[m_pObject]), "fragment"));
+		if (need_update) {
+			gnome_canvas_pango_set_selection_bounds (m_Active,  bounds.cur,  bounds.cur);
+			pFragment->AnalContent ((unsigned) bounds.start, (unsigned&) bounds.cur);
+			pFragment->OnChanged (false);
+		}
 		m_pView->SetGnomeCanvasPangoActive (m_Active);
 		g_object_set (G_OBJECT (m_Active), "editing", true, NULL);
 		m_CurNode = ((gcpFragment*) m_pObject)->SaveSelected ();
